/*====================================================
=            GULP FILE - REQUIRED FOR DEV            =
====================================================*/

// Importing packages installed
var devBuild = true;

const gulp = require('gulp'); // Gulp - The task Runner
const uglify = require('gulp-uglify'); // Minify JS
const clean = require('rimraf'); // Clean / Delete
const concat = require('gulp-concat'); // Combine files (e.g all js files into one)
const rename = require('gulp-rename'); // Add all src data output to new file
const sass = require('gulp-sass'); // Sass compiler
const minifyCss = require('gulp-minify-css'); // Minify CSS
const autoprefix = require('gulp-autoprefixer'); // AUtoprefix CSS
const autoprefixer = require('autoprefixer');
const plumber = require('gulp-plumber'); // Stopping gulp from crashing once errors
const imagemin = require('gulp-imagemin'); // For Image Compression
const browserSync = require('browser-sync'); // Live browser sync
const cache = require('gulp-cached'); // Caching already loaded files
const svgmin = require('gulp-svgmin');
const mainBowerFiles = require('gulp-main-bower-files');
const filter = require('gulp-filter');
const order = require("gulp-order");
const eventStream = require('event-stream');
const cssnano = require('cssnano');
const phplint = require('gulp-phplint');
const scsslint = require('gulp-scss-lint');
const reload = browserSync.reload; // Reloading var for browser sync
const eslint = require('gulp-eslint');
const babel = require('gulp-babel');
const noop = require('gulp-noop');
const postcss = require('gulp-postcss');
const assets = require('postcss-assets');

const stripdebug = require('gulp-strip-debug');
const sourcemaps = require('gulp-sourcemaps');

var sassPaths = [
	'bower_components/foundation-sites/scss',
	'bower_components/motion-ui/src',
	'./assets/src/styles'
];

/*===============================
=            SCRIPTS            =
===============================*/
/*
	Get JS, Stop erros from breaking Gulp, add all js into one file,
	minify js, name new file & add .min to name, add to new file,
	Reload browser
*/

gulp.task('scripts', function(){

	var jsFilter = filter('**/*.js');

	//Combine all required Bower Files
	var vendorFiles = gulp.src(['./bower.json'])
		.pipe(mainBowerFiles())
		.pipe(jsFilter)
		.pipe(concat('vendor.js'));

	//Combine all required App Files
	var appFiles = gulp.src(['assets/src/scripts/**/*.js'])
		.pipe(babel({
			presets: ['@babel/preset-env']
		}))
		.pipe(concat('app.js'))
		.pipe(devBuild ? noop() : stripdebug());

	//Merge Vendor & App Files
	return eventStream.concat(vendorFiles, appFiles)
		.pipe(order([
			"vendor.js",
			"app.js"
		]))
		.pipe(plumber())
		.pipe(concat('bundle.js'))
		.pipe(rename({suffix: '.min'}))
		.pipe(uglify())
		.pipe(gulp.dest('assets/dist/scripts'))
		.pipe(reload({stream: true}));
});

gulp.task('lint-js', () => {
	return gulp.src(['assets/src/scripts/**/*.js'])
		.pipe(cache('eslint'))
		// eslint() attaches the lint output to the "eslint" property
		// of the file object so it can be used by other modules.
		.pipe(eslint())
		// eslint.format() outputs the lint results to the console.
		// Alternatively use eslint.formatEach() (see Docs).
		.pipe(eslint.format())
		// To have the process exit with an error code (1) on
		// lint error, return the stream and pipe to failAfterError last.
		.pipe(eslint.failAfterError());
});

/*=================================
=            SASS TASKS           =
=================================*/
gulp.task('styles', function(){

	return gulp.src(['./assets/src/styles/*.scss'])
	.pipe(plumber())
	.pipe(devBuild ? sourcemaps.init() : noop())
	.pipe(sass({
		includePaths: sassPaths
	}).on('error', sass.logError))
	.pipe(postcss([
		assets({ loadPaths: ['./assets/dist/images/', './assets/dist/fonts/'], cachebuster: true, relative: true}),
		autoprefixer({ browsers: ['last 2 versions', '> 2%'] }),
		cssnano
	]))
	.pipe(devBuild ? sourcemaps.write('') : noop())
	.pipe(rename(function(path) {
		if (!path.extname.endsWith('.map')) {
			path.basename += '.min';
		}
	}))
	.pipe(gulp.dest('./assets/dist/styles'))
	.pipe(reload({stream: true}));

});

gulp.task('lint-scss', function() {

	const gulpStylelint = require('gulp-stylelint');

	return gulp.src(['./assets/src/styles/**/*.scss', '!assets/src/styles/globals/_settings.scss', '!./assets/src/styles/components/font-awesome/**/*.scss'])
	.pipe(plumber())
	.pipe(cache('lint-scss'))
	.pipe(gulpStylelint({
		reporters: [
			{formatter: 'string', console: true}
		]
	}));

});

/*====================================
=            FONTS - TASK            =
====================================*/
gulp.task('fonts', function() {
	return gulp.src(['./assets/src/fonts/*'])
	.pipe(plumber())
	.pipe(cache('fonts'))
	.pipe(gulp.dest('./assets/dist/fonts'));
});

/*=================================================
=            IMAGE TASKS - COMPRESSION            =
=================================================*/
gulp.task('images', function(){
	return gulp.src(['./assets/src/images/**/*'])
	.pipe(plumber())
	.pipe(cache('images'))
	.pipe(imagemin({ optimizationLevel: 5 }))
	.pipe(gulp.dest('./assets/dist/images'));
});

gulp.task('svg', function(){
	return gulp.src(['./assets/src/images/**/*.svg'])
	.pipe(plumber())
	.pipe(cache('images'))
	.pipe(svgmin())
	.pipe(gulp.dest('./assets/dist/images'));
});


/*========================================
=            FILES TASK - PHP            =
========================================*/
/*
	Get PHP Files (this is for watching for changes),
	Reload Browser
*/

gulp.task('phpFiles', function(){
	return gulp.src(['./**/*.php'])
	.pipe(plumber())
	.pipe(cache('phpfiles'))
/*
	.pipe(phplint('', { /*opts }))
	.pipe(phplint.reporter(function(file){
      var report = file.phplintReport || {};
      if (report.error) {
        console.error(report.message+' on line '+report.line+' of '+report.filename);
      }
    }))
*/
	.pipe(reload({stream: true}));
});


/*=========================================
=            BROWSER SYNC TASK            =
=========================================*/
// BrowserSYnc - For browser LIVE reload

gulp.task('browser-sync',  function(){
	browserSync.init({
		scrollProportionally: true,
		//logLevel: "debug", // For debugging
		injectChanges: true,
		port: 3004,
		open: false, // Set to true to open automatically on local enviroment
		proxy: {
			target: "https://stock.mct.dev.wearecoast.com.au",
		},
		https: {
			key: "/etc/letsencrypt/live/mct.dev.wearecoast.com.au/privkey.pem",
			cert: "/etc/letsencrypt/live/mct.dev.wearecoast.com.au/cert.pem"
		}
	});
});


/*==================================
=           CLEAN TASKS            =
==================================*/
// Clean out the dist directory

gulp.task('clean-fonts', [], function(cb) {
	clean('./assets/dist/fonts', cb);
});

gulp.task('clean-images', [], function(cb) {
	clean('./assets/dist/images', cb);
});

gulp.task('clean-scripts', [], function(cb) {
	clean('./assets/dist/scripts', cb);
});

gulp.task('clean-styles', [], function(cb) {
	clean('./assets/dist/styles', cb);
});


/*==================================
=            WATCH TASK            =
==================================*/
// Watch for changes on all file based tasks above

gulp.task('watch', function(){
	gulp.watch(['assets/src/scripts/**/*.js'], ['scripts']);
	gulp.watch('assets/src/styles/**/*.scss', ['styles']);
	gulp.watch('assets/src/fonts/**/*', ['fonts']);
	gulp.watch('assets/src/images/**/*', ['images', 'svg']);
	gulp.watch('./**/*.php', ['phpFiles']);
});

gulp.task('initBuild', function() {
	devBuild = false;
});

/*=====================================
=            DEFAULT TASKS            =
=====================================*/

var fs = require('fs');
if (1) fs.chmod = function (a, b, cb) {
	cb(0);
}

// Setup the default task to run all tasks with simply writing gulp, into the command prompt.
gulp.task('default', ['clean-fonts', 'clean-images', 'styles', 'scripts', 'fonts', 'images', 'phpFiles', 'browser-sync', 'watch']);

gulp.task('build', ["initBuild", "clean-fonts", "clean-images", "clean-styles", "clean-scripts", "styles", "images", "scripts", "fonts"]);
